#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include "common.h"
#include "CreateTFParam.h"
#include "HarkioParam.h"
#include "transform.h"
#include "GnuPlot.h"
#include "PlotControl.h"
#include "msgdialog.h"
#include "TemplateControl.h"
#include "helpdialog.h"


namespace Ui {
class MainWindow;
}





class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();


public:
    int             m_in_page_no;  //現在開いている inputページ番号
    int             m_out_page_no; //現在開いている oututページ番号
    int             m_run;         //現在実行状態
    int             m_kind;        //実行中プロセス種別
    int             m_bar;         //プログレスバーのvalue
    CreateTFParam   m_param;
    Transform       m_transform;
    GNUPlot         m_gnuplot;
    PlotControl     m_plotctl[256];
    QString         m_LastPath;
    QProcess        m_Process;
    MsgDialog       m_msgdialog;
    TemplateControl m_hark_file_path;
    HelpDialog      m_helpdialog;
    boost::filesystem::path  m_convfile;   //text<->xml 変換ファイルパス

    //   std::stringstream m_filelist[8]; //file list一時格納領域


public:
    void IMPcreateTemplate();   //impulus template作成
    void TSPcreateTemplate();   //TSP     template作成
    void MICcreateTemplate();   //mic template作成
    void NOISEcreateTemplate(); //noise template作成
    void M2PGcreate();          //M2PG 作成
    void M2PGDataDisp();        //M2PGcreateで作成したharkioを表示する
    void GTFcreate();           //GTF 作成
    void GTFDataDisp();         //GTFcreateで作成したharkioを表示する
    void M2PGDispClear();       //M2PG結果表示画面クリア
    void GTFDispClear();        //GTF結果表示画面クリア
    std::string harktoolDir();  //
    void setTopLabel(int kind);
    void ButtonClearFocus();
	void tmp_dir_delete(QString strPath);
	int  CalcNumelv(double elvfrom , double elvto, double elvint);
	int  CalcNumheight(  double heightfrom , double heightto, double heightint);
	int  CalcNumazi(double azifrom, double azito, double aziint);
    void ListFileEdit();
    void TSPconvXmlToText();
    void TSPconvTextToXml();
    void IMPconvXmlToText();
    void IMPconvTextToXml();


    //-----------------------------------------//
    //伝達関数plot用
    //-----------------------------------------//
    void dispGraph();
    void setHarkioParam();       //harkioのデータをharkioparam.hに格納
    int  otherUnit(int target, int x_unit, int y_unit);
    int  GetPlotTitle(int target, int x_unit, int y_unit);
    void setIndex(std::vector<int> *indexes, int xyz_axis, int specific_axis);
    void createTFTitle(std::string &title);
    std::string zUnitName(int id);
    void initPlotLabel();                  //label 初期化
    void initPlotStyle();                  //plot style初期化
    int  presetTFGraph();                  //TFG前処理
    void removeComboAll(QComboBox *combo); //comboBox内の内容をすべて削除
    void setMicCombo();                    //mic番号をcombo boxに設定
    void setDispIndex(int index);          //disp_indexのcomboBoxに値を設定
    void setCheckLabel();                  //check label 設定
    void setCheckLabel_NG();
    void changeEvent(QEvent *e);
    void createIFFTTitle(std::string &title);
    int  presetIFFTGraph();
    void axis_reset();
    void plottype_reset();
    TFMatrix& IFFT(TFMatrix& matrix, int FFT, int NMic, int NSrc);//IFFT変換処理
    void InfoMsgBox(QString msg);
    void setRunStatus(int proc); //現在のpageを設定
    void setPageNo(int proc);    //page no を設定
    void setInit(int kind);
    void dispGraphTemplate(); //template グラフ表示処理
    int  RetInfoMsgBox(QString msg);
    void THROW_INFO(QString msg);
    void THROW_CRITICAL(QString msg);


private slots:
    void on_Button_IMP_clicked();
    void on_Button_TSP_clicked();
    void on_Button_MIC_clicked();
    void on_Button_NOISE_clicked();
    void on_button_Create_clicked();
    void on_tsp_file_button_clicked();
    void on_button_Localization_clicked();
    void on_tsp_spherical_clicked();
    void on_tsp_cylindrical_clicked();
    void on_tsp_org_Button_clicked();
    void on_imp_cylindrical_clicked();
    void on_imp_spherical_clicked();
    void on_mic_spherical_clicked();
    void on_mic_cylindrical_clicked();
    void on_m2pg_file_Button_clicked();
    void on_m2pg_list_Button_clicked();
    void on_m2pg_table_cellDoubleClicked(int row, int column);
    void on_exit_Button_clicked();
    void hark_strat();
    void hark_close();
    void ProcFinished(int exitCode, QProcess::ExitStatus exitStatus);
    void ProcOutput();                               //プログレスバー表示処理
    void ProcError();
    void ErrorMsg();
    void on_Button_Graph_clicked();
    void on_button_Separation_clicked();
    void on_color_map_currentIndexChanged(int index);
    void on_x_combo_currentIndexChanged(int index); //  x軸　comboboxの値変更時の振り分け処理
    void on_y_combo_currentIndexChanged(int index);
    void ChangeComboBOX_TF(int x_index , int y_index);//comboBoxが変更されたとき
    void ChangeComboBOX_IFFT(int x_index , int y_index);
    void on_target_currentIndexChanged(int index);// target comboboxが変更されたときの処理
    void on_reset_param_clicked();
    void on_reset_label_clicked();
    void setTextLabel(QString label , QString text );
    void setTimeCombo() ;
    void setFreqCombo() ;
    void setSrcPosCombo() ;
    void on_gtf_file_Button_clicked();
    void on_gtf_list_Button_clicked();
    void on_gtf_table_cellDoubleClicked(int row, int column);
    void on_gtf_table_cellChanged(int row, int column);
    void on_action_open_triggered();
    void on_action_save_triggered();
    void on_action_8_triggered();
    void on_Button_Clear_clicked();
    void on_checkBox_1_clicked();
    void on_checkBox_2_clicked();
    void on_checkBox_3_clicked();
    void on_checkBox_4_clicked();
    void on_m2pg_table_cellPressed(int row, int column);

    void on_gtf_table_cellPressed(int row, int column);

    void on_imp_file_button_clicked();

    void on_action_Exit_triggered();

    void on_action_Clear_triggered();

    void on_action_Plot_triggered();

    void on_action_Create_triggered();

    void on_action_Separation_triggered();

    void on_action_Localization_triggered();

    void on_action_Noise_triggered();

    void on_action_MicArray_triggered();

    void on_action_Tsp_triggered();

    void on_action_Implus_triggered();

    void on_Button_save2_clicked();

    void on_action_XML_TEXT_triggered();

    void on_arrow_button_2_clicked();

    void on_arrow_button_1_clicked();


private:
    Ui::MainWindow *ui;
};



#endif // MAINWINDOW_H
