#include "m2pgcreate.h"
#include "mainwindow.h"
#include "ui_mainwindow.h"




M2PGcreate::M2PGcreate(QWidget *parent) :
    QMainWindow(parent)
{


}




//-------------------------------------------------------------------//
// 定位伝達関数作成
//-------------------------------------------------------------------//
void MainWindow::M2PGcreate()
{

    try
    {

        qDebug() << __func__;

        //画面クリア
        M2PGDispClear();


        // 数字パラメータ格納
        m_param.NFFT                 = ui->m2pg_fft_length->value();    //FFT長
        m_param.NFS                  = ui->m2pg_sampling_freq->value(); //サンプリング周波数
        m_param.DIRECT_LENGTH        = ui->m2pg_direct_length->value(); //直接音長
        m_param.IMPULSE_RESPONSE_LEN = ui->m2pg_tsp_length->value();    //tsp長
        m_param.geometry_calculus    = ui->m2pg_geometry->isChecked();  //幾何計算


        qDebug() << "m2pg_geometry->isChecked()=" << m_param.geometry_calculus ;


        if(m_plotctl[m_out_page_no].harkio != NULL)
        {
            delete m_plotctl[m_out_page_no].harkio;
            m_plotctl[m_out_page_no].harkio = NULL;

        }

        m_plotctl[m_out_page_no].harkio = new harkio::HARK;



        //-------------------------------------------------------------------//
        // MICARY-LocationFile のチェック
        //-------------------------------------------------------------------//
        qDebug() << "MICARY-LocationFile のチェック";

        std::string fname;
        fname = ui->m2pg_mic_file->text().toStdString();

        qDebug() << "fname=" << fname.c_str() ;


        if (fname == "")
        {
               THROW_CRITICAL(tr("Input MICARY-LcationFile", "MICARY-LcationFileを入力してください"));
                return;
        }


        auto mic_file_type = GetHarkConfigFileType(fname);


        if (mic_file_type.second == HarkConfigFileType::Unknown)
        {
           THROW_CRITICAL(tr("Unknown format:MicArrayLcationFile", "MicArrayLcationFileが不明なフォーマットです"));
           return;
        }
        else
        {
            if (mic_file_type.second != HarkConfigFileType::MicArrayLocationFile)
            {
                THROW_CRITICAL(tr("Invalid file type:MicArrayLcationFile", "MicArrayLcationFileが不正なファイルタイプです"));
                return;
            }
        }


        if (mic_file_type.first != HarkConfigFileFormatType::XML)
        {
            THROW_CRITICAL("Not XML file:MicArrayLcationFile");
            return;
        }


        m_param.mic_file = fname; // MicArrayLcationFileを格納


        qDebug()  << "param.geometry_calculus=" << m_param.geometry_calculus ;


        //-------------------------------------------------------------------//
        // ImpulseResponse-ListFIle, TSP-ListFile のチェック
        //-------------------------------------------------------------------//
        fname = ui->m2pg_list_file->text().toStdString();

        qDebug() << "fname=" << fname.c_str() ;


        if (fname == "")
        {
            THROW_CRITICAL(tr("Input TSP/IRListFile", "TSP/IRListFileを入力してください"));
            return;
        }


        //ファイルタイプ確認
        auto src_file_type = GetHarkConfigFileType(fname);


        if (src_file_type.second == HarkConfigFileType::Unknown)
        {
            THROW_CRITICAL(tr("Unknown format:%1", "%1が不明なフォーマットです").arg("TSP/IRListFile"));
            return;
        }
        else if (src_file_type.first != mic_file_type.first)
        {
            THROW_CRITICAL(tr("MicArrayLocationFile and %1 have to be the same format (Text or XML)",
                                "MicArrayLocationFileと%1は同じフォーマットでなければいけません (Text または XML)" )
                    .arg("TSP/IRListFile"));
            return;
        }
        else if (src_file_type.second != HarkConfigFileType::ImpulseResponseListFile &&
                 src_file_type.second != HarkConfigFileType::TSPListFile)
        {
            THROW_CRITICAL(tr("Invalid file type:%1", "%1が不正なファイルタイプです").arg("TSP/IRListFile"));
            return;
        }

        //file type XMLチェック
        if (src_file_type.first != HarkConfigFileFormatType::XML)
        {
            THROW_CRITICAL("Not XML file:TSP/IRListFile");
            return;
        }


        m_param.src_file = fname ; // IR/TSP listファイル設定


        // 一時ディレクトリを生成
        boost::filesystem::path temp_dir = CreateTemporaryDirectory();
        boost::filesystem::path hark_file_path = temp_dir / "hark.tf";


        printf("*---------------------------------------------------------*\n");

        printf("harktool4_cui \n"
                            "--mode tf \n"
                            "--output-file %s \n"
                            "--output-format binary \n"
                            "--output-type m2pg \n"
                            "--nfft %d \n"
                            "--normalize-mic \n"
                            "--sampling-freq %d \n"
                            "%s \n"   // geometry_calculus
                            " %s \n"   // mic_file
                            " %s \n"   // src_file
                            , "hark.tf"
                            ,m_param.NFFT
                            ,m_param.NFS
                            ,(m_param.geometry_calculus ? "--geometry-calculus 1" : "")
                            , m_param.mic_file.c_str()
                            , m_param.src_file.c_str() );

        printf("*---------------------------------------------------------*\n");


        QString command = (boost::format("harktool4_cui "
                                                        "--mode tf "
                                                        "--output-file %s "
                                                        "--output-format binary "
                                                        "--output-type m2pg "
                                                        "--nfft %d "
                                                        "--normalize-mic "
                                                        "--sampling-freq %d "
                                                        "%s "
                                                        "%s "
                                                        "%s " )
                                                        % hark_file_path
                                                        % m_param.NFFT
                                                        % m_param.NFS
                                                        % (m_param.geometry_calculus ? "--geometry-calculus 1" : "")
                                                        % m_param.mic_file
                                                        % m_param.src_file).str().c_str();


        qDebug() << "command = " << command ;


        //run status 設定
        setRunStatus(RUN_MUSIC_CUI);

        m_Process.start(command);//harktool4_cui実行

        qDebug() << "m_Process.pid=" << m_Process.pid();
        qDebug() << "hark_file_path=" << hark_file_path.native().c_str() ;

        m_plotctl[m_out_page_no].hark_file_path = hark_file_path;

        MainWindow::m_bar = 0;
        ui->progressBar->setValue(0);
        ui->progressBar->setRange(0,72);

        ui->stackedWidget->setCurrentIndex(DISP_scrProgressBar);//表示

        setPageNo(RUN_MUSIC_CUI);//表示画面番号格納

        MainWindow::setCheckLabel();//chek中label表示

        QTimer::singleShot( 5000, this, SLOT(ErrorMsg()) );

        qDebug("CalcSys end");


    }
    catch (const std::exception& e)
    {
         THROW_CRITICAL(e.what());
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("unexpected error"));
    }

}






//----------------------------------------//
//  伝達関数計算結果を表示
//----------------------------------------//
void MainWindow::M2PGDataDisp()
{

    try
    {

        // 作成した伝達関数を読み込む
        qDebug() << "作成した伝達関数を読み込む";

        qDebug()  << "hark_file_path=" << m_plotctl[m_out_page_no].hark_file_path.native().c_str() ;


        boost::uintmax_t size = boost::filesystem::file_size(m_plotctl[m_out_page_no].hark_file_path.native().c_str());

        qDebug() << "File size= " << size ;

        if(size <=0 )
        {
            MainWindow::THROW_CRITICAL("Can not Create harkfile");
            return;
        }


        m_plotctl[m_out_page_no].harkio->readFile(m_plotctl[m_out_page_no].hark_file_path.native()); //harkio::HARKに格納



        // SPECデータの上書き
        harkio::SPEC* spec = m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::SPEC> ();
        spec->setDate(QDateTime::currentDateTime().toString("yyyy/MM/dd").toStdString().c_str());
        spec->setCreator(getUserName().toStdString().c_str());





        //-----------------------------------------------------------------//
        // 結果表示
        //-----------------------------------------------------------------//
        //編集不可に設定
        ui->m2pg_table->setEditTriggers(QAbstractItemView::NoEditTriggers);



        //------------------------------------------------------//
        // fmt chunk
        //------------------------------------------------------//
        QTableWidgetItem* item;
        QString value;
        int row = 0; //fmt開始行

        harkio::FMT *fmt = m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::FMT>();

        //fmtバージョン
        value = QString("%1").arg(fmt->getVersionStr().c_str());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white); //文字色
        item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
        ui->m2pg_table->setItem(++row,1,item);//1行,1列書き込み



        //Format ID
        value = QString("%1").arg(fmt->getFormatId());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white);
        item->setFont(QFont("Ubuntu",12,QFont::Bold));
        ui->m2pg_table->setItem(++row,1,item);


        //mic数
        value = QString("%1").arg(fmt->getNMic());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white);
        item->setFont(QFont("Ubuntu",12,QFont::Bold));
        ui->m2pg_table->setItem(++row,1,item);




        //FFT長
        value = QString("%1").arg(fmt->getNFFT());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white);
        item->setFont(QFont("Ubuntu",12,QFont::Bold));
        ui->m2pg_table->setItem(++row,1,item);


        //インパルスレスポンス長
        value = QString("%1").arg(fmt->getNIR());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white);
        item->setFont(QFont("Ubuntu",12,QFont::Bold));
        ui->m2pg_table->setItem(++row,1,item);


        //サンプリング周波数
        value = QString("%1").arg(fmt->getNFS());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white);
        item->setFont(QFont("Ubuntu",12,QFont::Bold));
        ui->m2pg_table->setItem(++row,1,item);


        //Mic Position
        item = new QTableWidgetItem("Plot");
        item->setForeground(Qt::white);
        item->setFont(QFont("Ubuntu",12,QFont::Bold));
        ui->m2pg_table->setItem(++row,1,item);


        ++row; //空白行分
        ++row; //M2PG表題分

        //------------------------------------------------------//
        // M2PG chunk
        //------------------------------------------------------//
        harkio::M2PG *m2pg = m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::M2PG>();



        //m2pgバージョン
        value = QString("%1").arg(m2pg->getVersionStr().c_str());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white); //文字色
        item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
        ui->m2pg_table->setItem(++row,1,item);//1行,1列書き込み


        //Num of Azimuth
        value = QString("%1").arg(m2pg->getND());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white); //文字色
        item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
        ui->m2pg_table->setItem(++row ,1,item);//1行,1列書き込み

        //距離の要素数
        value = QString("%1").arg(m2pg->getNR());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white); //文字色
        item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
        ui->m2pg_table->setItem(++row,1,item);//1行,1列書き込み


        //Source Location
        item = new QTableWidgetItem("Plot");
        item->setForeground(Qt::white);
        item->setFont(QFont("Ubuntu",12,QFont::Bold));
        ui->m2pg_table->setItem(++row,1,item);


        //Trancfer Function
        item = new QTableWidgetItem("Plot");
        item->setForeground(Qt::white);
        item->setFont(QFont("Ubuntu",12,QFont::Bold));
        ui->m2pg_table->setItem(++row,1,item);


        ++row; //空白行分
        ++row; //SPEC表題分

        //------------------------------------------------------//
        // SPEC chunk
        //------------------------------------------------------//

        //作成日
        value = QString("%1").arg(spec->getDate());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white); //文字色
        item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
        ui->m2pg_table->setItem(++row,1,item);//1行,1列書き込み


        //場所
        value = QString("%1").arg(spec->getLocation());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white); //文字色
        item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
        ui->m2pg_table->setItem(++row,1,item);//1行,1列書き込み


        //ロボット
        value = QString("%1").arg(spec->getRobot());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white); //文字色
        item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
        ui->m2pg_table->setItem(++row,1,item);//1行,1列書き込み

        //アレイのタイプ
        value = QString("%1").arg(spec->getArrayType());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white); //文字色
        item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
        ui->m2pg_table->setItem(++row,1,item);//1行,1列書き込み


        //作成者
        value = QString("%1").arg(spec->getCreator());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white); //文字色
        item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
        ui->m2pg_table->setItem(++row,1,item);//1行,1列書き込み


        //備考
        value = QString("%1").arg(spec->getMemo());
        item = new QTableWidgetItem(value);
        item->setForeground(Qt::white); //文字色
        item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
        ui->m2pg_table->setItem(++row,1,item);//1行,1列書き込み



        //------------------------------------------------------//
        // harkioの値をGNUPlot.hに設定する
        //------------------------------------------------------//
        m_plotctl[m_out_page_no].param.NMic =  fmt->getNMic();  //mic数
        m_plotctl[m_out_page_no].param.NFFT =  fmt->getNFFT(); // FFT長
        m_plotctl[m_out_page_no].param.NIR  =  fmt->getNIR();  // インパルスレスポンス長
        m_plotctl[m_out_page_no].param.NFS  =  fmt->getNFS();  //サンプリング周波数

        m_plotctl[m_out_page_no].param.M2PG_ND =  m2pg->getND();
        m_plotctl[m_out_page_no].param.M2PG_NR =  m2pg->getNR();



       //-----------------------------------------------------------------//
       // テンポラリdir削除
       //-----------------------------------------------------------------//
       if ( QFile::exists(  m_plotctl[m_out_page_no].hark_file_path.native().c_str() ) )
       {
           qDebug() << "テンポラリ削除 : " <<  m_plotctl[m_out_page_no].hark_file_path.native().c_str() ;

           QString str = m_plotctl[m_out_page_no].hark_file_path.native().c_str();

           if( str.indexOf("hark.tf") != -1)
           {
               str = str.left(str.indexOf("/hark.tf"));
               qDebug()  << "str=" << str;

               std::string temp = (boost::format("rm -rf %s") %  str.toStdString().c_str() ).str();
               ::system(temp.c_str());
               m_plotctl[m_out_page_no].hark_file_path = "";
            }
       }


    }
    catch (const std::exception& e)
    {
         THROW_CRITICAL(e.what());
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("unexpected error"));
    }
}



//----------------------------------------//
// harkio表示画面をclear
//----------------------------------------//
void MainWindow::M2PGDispClear()
{

    int i;

    //FMTチャンク初期化
    for( i = DISP_M2PG_FMT_VERSION ; i <= DISP_M2PG_MIC_POSITION ; i++)
    {
         ui->m2pg_table->setItem( i,1, new QTableWidgetItem(""));//空白
    }

    //M２PGチャンク初期化
    for( i = DISP_M2PG_VERSION ; i <= DISP_M2PG_TRANSFER_FUNCTION ; i++)
    {
         ui->m2pg_table->setItem( i ,1, new QTableWidgetItem(""));//空白
    }

    //SPECチャンク初期化
    for( i = DISP_M2PG_DATA ; i <= DISP_M2PG_NOTES ; i++)
    {
         ui->m2pg_table->setItem( i ,1, new QTableWidgetItem(""));//空白
    }

}



